@@ -9,12 +9,11 @@
 import logging
 import os
 import sys
-import re
 from ctypes import CDLL, CFUNCTYPE, POINTER, Structure, c_char_p
 
-from django.contrib.gis.geos.error import GEOSException
 from django.core.exceptions import ImproperlyConfigured
-from django.utils.functional import SimpleLazyObject
+from django.utils.functional import SimpleLazyObject, cached_property
+from django.utils.version import get_version_tuple
 
 logger = logging.getLogger('django.contrib.gis')
 
@@ -40,13 +39,14 @@ def load_geos():
         _lgeos = CDLL(libname)
     else:
         raise Exception('No valid platform name. Found {}'.format(os.name))
-
     # Here we set up the prototypes for the initGEOS_r and finishGEOS_r
     # routines.  These functions aren't actually called until they are
     # attached to a GEOS context handle -- this actually occurs in
     # geos/prototypes/threadsafe.py.
     _lgeos.initGEOS_r.restype = CONTEXT_PTR
     _lgeos.finishGEOS_r.argtypes = [CONTEXT_PTR]
+    # Set restype for compatibility across 32 and 64-bit platforms.
+    _lgeos.GEOSversion.restype = c_char_p
     return _lgeos
 
 
@@ -108,18 +108,10 @@ CS_PTR = POINTER(GEOSCoordSeq_t)
 CONTEXT_PTR = POINTER(GEOSContextHandle_t)
 
 
-# Used specifically by the GEOSGeom_createPolygon and GEOSGeom_createCollection
-#  GEOS routines
-def get_pointer_arr(n):
-    "Gets a ctypes pointer array (of length `n`) for GEOSGeom_t opaque pointer."
-    GeomArr = GEOM_PTR * n
-    return GeomArr()
-
-
 lgeos = SimpleLazyObject(load_geos)
 
 
-class GEOSFuncFactory(object):
+class GEOSFuncFactory:
     """
     Lazy loading of GEOS functions.
     """
@@ -127,21 +119,22 @@ class GEOSFuncFactory(object):
     restype = None
     errcheck = None
 
-    def __init__(self, func_name, *args, **kwargs):
+    def __init__(self, func_name, *args, restype=None, errcheck=None, argtypes=None, **kwargs):
         self.func_name = func_name
-        self.restype = kwargs.pop('restype', self.restype)
-        self.errcheck = kwargs.pop('errcheck', self.errcheck)
-        self.argtypes = kwargs.pop('argtypes', self.argtypes)
+        if restype is not None:
+            self.restype = restype
+        if errcheck is not None:
+            self.errcheck = errcheck
+        if argtypes is not None:
+            self.argtypes = argtypes
         self.args = args
         self.kwargs = kwargs
-        self.func = None
 
     def __call__(self, *args, **kwargs):
-        if self.func is None:
-            self.func = self.get_func(*self.args, **self.kwargs)
         return self.func(*args, **kwargs)
 
-    def get_func(self, *args, **kwargs):
+    @cached_property
+    def func(self):
         from django.contrib.gis.geos.prototypes.threadsafe import GEOSFunc
         func = GEOSFunc(self.func_name)
         func.argtypes = self.argtypes or []
@@ -151,28 +144,11 @@ class GEOSFuncFactory(object):
         return func
 
 
-# Returns the string version of the GEOS library. Have to set the restype
-# explicitly to c_char_p to ensure compatibility across 32 and 64-bit platforms.
-geos_version = GEOSFuncFactory('GEOSversion', restype=c_char_p)
-
-# Regular expression should be able to parse version strings such as
-# '3.0.0rc4-CAPI-1.3.3', '3.0.0-CAPI-1.4.1', '3.4.0dev-CAPI-1.8.0' or '3.4.0dev-CAPI-1.8.0 r0'
-version_regex = re.compile(
-    r'^(?P<version>(?P<major>\d+)\.(?P<minor>\d+)\.(?P<subminor>\d+))'
-    r'((rc(?P<release_candidate>\d+))|dev)?-CAPI-(?P<capi_version>\d+\.\d+\.\d+)( r\d+)?$'
-)
+def geos_version():
+    """Return the string version of the GEOS library."""
+    return lgeos.GEOSversion()
 
 
-def geos_version_info():
-    """
-    Returns a dictionary containing the various version metadata parsed from
-    the GEOS version string, including the version number, whether the version
-    is a release candidate (and what number release candidate), and the C API
-    version.
-    """
-    ver = geos_version().decode()
-    m = version_regex.match(ver)
-    if not m:
-        raise GEOSException('Could not parse version info string "%s"' % ver)
-    return {key: m.group(key) for key in (
-        'version', 'release_candidate', 'capi_version', 'major', 'minor', 'subminor')}
+def geos_version_tuple():
+    """Return the GEOS version as a tuple (major, minor, subminor)."""
+    return get_version_tuple(geos_version().decode())
